﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Ahp
{
    /// <summary>
    /// Represents AHP hierarchy
    /// </summary>
    public class Hierarchy
    {
        private GoalNode goalNode;
        /// <summary>
        /// Root node of the hierarchy
        /// </summary>
        public GoalNode GoalNode
        {
            get { return goalNode; }
        }

        private AlternativeCollection alternatives;
        /// <summary>
        /// Collection of alternatives
        /// </summary>
        public AlternativeCollection Alternatives
        {
            get { return alternatives; }
        }

        public Hierarchy()
            : this("Goal")
        { }

        public Hierarchy(string goalName)
        {
            goalNode = new GoalNode(goalName);
            alternatives = new AlternativeCollection(HandleAlternativeAdded, HandleAlternativeRemoved);
        }

        /// <summary>
        /// Returns AlternativeNode object by Alternative and CriterionNode objects
        /// </summary>
        /// <param name="alternative"></param>
        /// <param name="criterion"></param>
        /// <returns></returns>
        public AlternativeNode this[Alternative alternative, CriterionNode criterion]
        {
            get
            {
                if (!this.Alternatives.Contains(alternative))
                {
                    throw new KeyNotFoundException(String.Format("Alternative '{0}' not found in the Altenatives collection of the hierarchy", alternative.Name));
                }

                CriterionNodeCollection lowestCriterionNodes = this.GoalNode.GetLowestCriterionNodes();
                if (!lowestCriterionNodes.Contains(criterion))
                {
                    throw new KeyNotFoundException(String.Format("Criterion node '{0}' not found in the criterion nodes which are at lowest level of the hierarchy", criterion.Name));
                }

                RefreshAlternativeNodes();

                foreach (CriterionNode lowestCriterionNode in lowestCriterionNodes)
                {
                    foreach (AlternativeNode alternativeNode in lowestCriterionNode.AlternativeNodes)
                    {
                        if (Object.ReferenceEquals(lowestCriterionNode, criterion) &&
                            Object.ReferenceEquals(alternativeNode.Alternative, alternative))
                        {
                            return alternativeNode;
                        }
                    }
                }

                throw new ArgumentOutOfRangeException("AlternativeNode object for specified Alternative and CriterionNode objects not found");
            }
        }

        public void RefreshAlternativeNodes()
        {
            CriterionNodeCollection lowestCriterionNodes = this.GoalNode.GetLowestCriterionNodes();
            foreach (CriterionNode lowestCriterionNode in lowestCriterionNodes)
            {
                foreach (Alternative alternative in this.Alternatives)
                {
                    if (!lowestCriterionNode.AlternativeNodes.ContainsWithAlternative(alternative))
                    {
                        lowestCriterionNode.AlternativeNodes.Add(new AlternativeNode(alternative));
                    }
                }

                AlternativeNodeCollection toBeRemoved = new AlternativeNodeCollection();
                foreach (AlternativeNode alternativeNode in lowestCriterionNode.AlternativeNodes)
                {
                    if (!this.Alternatives.Contains(alternativeNode.Alternative))
                    {
                        toBeRemoved.Add(alternativeNode);
                    }
                }

                foreach (AlternativeNode alternativeNode in toBeRemoved)
                {
                    lowestCriterionNode.AlternativeNodes.Remove(alternativeNode);
                }
            }
        }

        private void HandleAlternativeAdded(Alternative alternative)
        {
            foreach (CriterionNode lowestCriterionNode in goalNode.GetLowestCriterionNodes())
            {
                lowestCriterionNode.AlternativeNodes.Add(new AlternativeNode(alternative));
            }
        }

        private void HandleAlternativeRemoved(Alternative alternative)
        {
            foreach (CriterionNode lowestCriterionNode in goalNode.GetLowestCriterionNodes())
            {
                AlternativeNode toBeDeleted = lowestCriterionNode.AlternativeNodes[alternative];
                lowestCriterionNode.AlternativeNodes.Remove(toBeDeleted);
            }
        }
    }
}
